<?php

// Burp  - Bart's User Registration Pages
//
// Version 2.0
// 
// Usage
// 		Burp is designed to be simple in use. To use it, just include the burp.inc.php file
// 		in your PHP script and program your PHP around it.
//
// User functions exported by the Burp library:
// burp_start(bool login_required)
//		Call this to start any Burp session. Using the 'login_required' boolean, you can
//		define if a login screen will show if the user is not logged in. Set it to false
//		for pages that will have different/extra functionality when logged in. 
//
// burp_stop()
//		Call this to stop a Burp session. Is analogous to a logout. Note that you have to 
//		call burp_start() first to initialise the session (you cannot stop an uninitialised 
//		session.
//
// burp_logged_in()
//		This will return true when a user is logged in.
//
// burp_username()
//		This will return the currently logged in user, "" when nobody is logged in.
//
// burp_user_id()
//		This will return the id of the currently logged in user, 0 when nobody logged in,
//		or -1 if the burp_allowed_user array was used.
//
// For now, only MySQL is tested, but Burp is written to be database agnostic and has its
// own query functions. The table should *at least* look like this (you are free to add any
// fields that you might need, use burp_user_id() to get the user id):
// mysql> describe burp_users;
// +-------------+------------------+------+-----+---------+----------------+
// | Field       | Type             | Null | Key | Default | Extra          |
// +-------------+------------------+------+-----+---------+----------------+
// | id          | int(10) unsigned | NO   | PRI | NULL    | auto_increment | 
// | username    | varchar(64)      | NO   | UNI | NULL    |                | 
// | password    | char(40)         | NO   |     | NULL    |                |
// | permissions | text             | NO   |     | NULL    |                |
// | timestamp   | datetime         | NO   |     | NULL    |                | 
// +-------------+------------------+------+-----+---------+----------------+
// 4 rows in set (0.01 sec)
//
// Create with:
/*
 CREATE TABLE `burp_users` (
  `id` int(10) unsigned NOT NULL auto_increment,
  `username` varchar(64) NOT NULL,
  `password` char(40) NOT NULL,
  `email` varchar(200) NOT NULL,
  `permissions` text NOT NULL,
  `timestamp` datetime NOT NULL,
  `status` enum('active','inactive') NOT NULL,
  `activationcode` char(8) NOT NULL,
  PRIMARY KEY  (`id`),
  UNIQUE KEY `username` (`username`)
) 
*/
// Notes:
// - The 'password' field is 40 bytes long, which fits an SHA1 hash, which is Burp's default. You can
//   change this if needed.
// - The 'username' field has a fairly arbitrary length limit. You can change it if needed.
// - The 'timestamp' field is updated every login.
// - Logging in, without logging out first, will replace the session.
// - There is some HTML and CSS spread around in the file:
//   - this way, burp can stay a single-file-application
//   - the HTML/CSS code is written as HERE files, so they are easy to edit
//   - some are in variables, so they can be overridden
//
// Changelog
// 2.0
// - Dropped 'allowed_user' array support (only MySQL support)
// - Implemented burp into the tbwb website framework
// - Burp uses Smarty templates
// - Burp 2.0 is not drop-in anymore 
// 1.1
// - Implemented burp_root_permission
// 1.0
// - Use MySQL if available, the $allowed_user array otherwise.
// - Use hashes for passwords, instead of plaintext, selectable with the 'burp_hash_function'.
// - The form uses POST now.
// - Burp uses global settings variables.
// 0.2
// - Fixed session code. It uses the $_SESSION array for reading the session variables.

//----------------------------- BEGIN OF USER DEFINABLES -------------------------------------

// User definables can be entered in this script for system-wide usage, or they can
// be overwritten before any burp call.

// Allowed users. The hash behind the name is a hash, generated
// with the 'hash' function (either in PHP or MySQL)
$burp_config['hash_function'] 			= "sha1";

// Logout auto relocate, when set to true, the burp_stop will logout and relocate
// the browser to $burp_index
$burp_config['logout_auto_relocate']	= true;

// This is where burp will jump to when login was successful.
// When set to $_SERVER['HTTP_REFERER'], Burp can be used to 
// login anywhere in the application.
$burp_config['index']					= $_SERVER['HTTP_REFERER'];

// if a user has the burp_root_permission set, the burp_permission() call 
// will always return true, no matter what permission is given
// use empty string ("") to disable this.
$burp_config['root_permission']			= "admin";

//----------------------------- END OF USER DEFINABLES -------------------------------------
$burp_config['version']					= "2.0";

define ('BURP_USER_OK',			0);
define ('BURP_USER_WRNGPASS',	1);
define ('BURP_USER_NOACTIVE', 	2);
define ('BURP_USER_ERR', 		3);
define ('BURP_CODE_NOTFOUND',	4);
define ('BURP_ACCT_ACTIVE',		5);
define ('BURP_ACCT_ACTIVATED',	6);
define ('BURP_USER_NOEXIST',	7);

// Includes
include (FS_ROOT . "lib/smarty/Smarty.class.php");

function logout_screen() {
	global $smarty;
	global $burp_config;

	$smarty->assign("index", $burp_config['index']);
		
	return $smarty->fetch(TPL_DIR . "logout.tpl");
}

function login_screen($message) {
	global $smarty;
	global $burp_config;
	
	$smarty->assign("index", $burp_config['index']);
	$smarty->assign("message", $message);
	
	return $smarty->fetch(TPL_DIR . "login.tpl");
}

function user_ok($username, $password) {
	global $burp_config;
	$ret = BURP_USER_ERR;

	$db_ok = false;
	$_SESSION['burp_user_id'] = 0;
	$_SESSION['burp_permissions'] = "";
	$_SESSION["burp_username"] = "";
	
	// check table
	$r = burp_query("SHOW TABLES LIKE 'burp_users'");
	if (burp_num_rows($r) != 0) {
		// yay! we found a table called burp_users, use it
		$db_ok = true;
	}

	if ($db_ok) {
		$username = burp_escape($username);
		$password = burp_escape($password);
		$r = burp_query("SELECT id, permissions, status FROM burp_users ".
			"WHERE username='$username' ". 
			"AND password={$burp_config['hash_function']}('$password')");
		
		if (burp_num_rows($r) == 0) {
			$ret = BURP_USER_WRNGPASS;
		} else {
			$row = burp_row($r);
			$status = $row['status'];
			
			if ($status == "active") {
				$_SESSION['burp_user_id'] = $row['id'];
				$_SESSION['burp_permissions'] = $row['permissions'];
				burp_query("UPDATE burp_users SET timestamp=NOW() WHERE id=".$row['id']);
				$ret = BURP_USER_OK;
			} else {
				$ret = BURP_USER_NOACTIVE;
			}
		}
	} else {
		user_error("Couldn't find 'burp_users' table.");
	}
	
	return $ret;
}

function burp_activate($activationcode) {
	$activationcode = burp_escape($activationcode);
	$r = burp_query ("SELECT id,username,status FROM burp_users WHERE activationcode='$activationcode' LIMIT 1");
	
	if (burp_num_rows($r) == 0) {
		// code not found
		$ret = BURP_CODE_NOTFOUND;
	} else {
		$row = burp_row($r);
		$status = $row['status'];
		
		if ($status == "active") {
			$ret = BURP_ACCT_ACTIVE;
		} else {
			// code found and not active yet, set active
			$id = $row['id'];
			burp_query ("UPDATE burp_users SET status='active' WHERE id=$id");
			$ret = BURP_ACCT_ACTIVATED;
		}
	}
	
	return $ret;
}

function burp_start($login_required = true) {
	global $allowed_user;
	
	// 1. See if we got a user/pass from a form
	if (isset($_POST['f_username']) && isset($_POST['f_password'])) {
		// check username/password pair
		$user_status = user_ok($_POST['f_username'], $_POST['f_password']); 
		
		if ($user_status == BURP_USER_OK) {
				// username exists and password is correct
				$_SESSION['burp_username'] = $_POST['f_username'];
		} else {
			switch ($user_status) {				
				case BURP_USER_WRNGPASS:
					$m = _s("Username or password incorrect.");
					break;
				case BURP_USER_NOACTIVE:
					$m = _s("Account is not activated.");
					break;
				default:
					$m = _s("Couldn't log in.");
					break;
			}
			
			$_SESSION['burp_username'] = "";
			echo login_screen($m);
			die();
		}
	} else if (@$_SESSION['burp_username'] == "" && $login_required) {
		// no form data received, show login form
		$_SESSION['burp_username'] = "";
		echo login_screen("");
		die();
	}
	
	// if we get here, we are logged in OK
	global $smarty;
	
	$__burp = array();
	$__burp['user'] = $_SESSION['burp_username'];
	
	$smarty->assign('burp', $__burp);
}

function burp_user_exists($username) {
	$username = burp_escape($username);
	$r = burp_query("SELECT username FROM burp_users WHERE username='$username'");
	
	return burp_num_rows($r) > 0;
}

function burp_logged_in() {
	return $_SESSION['burp_username'] != "";
}
function burp_username() {
	return $_SESSION['burp_username'];
}
function burp_user_id() {
	return $_SESSION['burp_user_id'];
}
function burp_permission($p) {
	$ok = false;
	$perms = explode(",", $_SESSION['burp_permissions']);
	foreach ($perms as $perm) {
		if ($p == $perm || ($perm == $burp_root_permission && $burp_root_permission != "")) {
			$ok = true;
			break;
		}
	}

	return $ok;
}
function burp_stop() {
	global $burp_config;

	// destroy login, but keep session
	$_SESSION['burp_username'] = "";
	
	if ($burp_config['logout_auto_relocate'] == true) {
		header("Location: ".$burp_config['index']);
	} else {
		echo logout_screen();
	}
	die();
}

function burp_add_user($user, $pass, $email) {
	global $burp_config;
	
	$user = burp_escape($user);
	$pass = burp_escape($pass);
	$email = burp_escape($email);
	
	// create an activation code
	$activationcode = "";
	for ($i = 0; $i <= 7; $i++) {
		$activationcode .= chr(mt_rand(ord('a'),ord('z')));
	}

	$r = burp_query ("INSERT INTO burp_users (username, password, email, permissions, timestamp, status, activationcode) ".
			"VALUES ('$user', ".$burp_config['hash_function']."('$pass'), '$email', 'guest', NOW(), 'inactive', '$activationcode')");

	return burp_affected_rows($r);
}

function burp_send_activation_code($username) {
	global $burp_config;
	
	$email = burp_get_email($username);
	$activationcode = burp_get_activationcode($username);
		
	if (mail_ok($email)) {
		global $smarty;
		$MIME_Boundary = "--------------REG_EMAIL_BOUNDARY";

		$smarty->assign("activationcode", $activationcode);
		$smarty->assign("MIME_Boundary", $MIME_Boundary);
		
		// address is valid, send mail
		$headers =
			"From: ".$burp_config['mail_from']."\r\n".
			"X-Mailer: Burp v".$burp_config['version']."\r\n".
			"MIME-Version: 1.0\r\n".
			"Content-Type: multipart/alternative; boundary=\"$MIME_Boundary\"\r\n";
			
		
		if (file_exists(TPL_DIR . "registration_email_".lc_current().".tpl")) {
			$tpl = "registration_email_".lc_current().".tpl";
		} else {
			// fallback to english
			$tpl = "registration_email_en_GB.tpl";
		}
		$message = $smarty->fetch($tpl);
		
		mail($email, _s("TBWB Service Website Registration"), $message, $headers);
	}
}

function burp_set_password($username, $password) {
	global $burp_config;

	$username = burp_escape($username);
	$password = burp_escape($password);
	
	$r = burp_query("UPDATE burp_users SET password=".$burp_config['hash_function']."('$password') ".
		"WHERE username='$username' LIMIT 1");
		
	if (burp_affected_rows($r) > 0) {
		$ret = BURP_USER_OK;
	} else {
		$ret = BURP_USER_NOEXIST;
	}
	
	return $ret;
}

function burp_get_email($username) {
	$username = burp_escape($username);
	$r = burp_query("SELECT email FROM burp_users WHERE username='$username'");
	$row = burp_row($r);
	
	return $row['email'];
}

function burp_get_activationcode($username) {
	$username = burp_escape($username);
	$r = burp_query("SELECT activationcode FROM burp_users WHERE username='$username'");
	$row = burp_row($r);
	
	return $row['activationcode'];
}

// BURP Database calls
// burp_num_rows should return a number of records, $r is whatever burp_query returned
function burp_num_rows($r) {
	return mysql_num_rows($r);
}
// burp_query() should return a database result
function burp_query($qstr) {
	return db_query($qstr);
}
// return a row from whatever burp_query returned in an assoc array
function burp_row($r) {
	return mysql_fetch_array($r, MYSQL_ASSOC);
}
// burp_affected_rows should return a number of updated records, $r is whatever burp_query returned
function burp_affected_rows($r) {
	// without parameters, mysql_affected_rows returns the last result
	return mysql_affected_rows();
}
// should escape a string for safe pasting into a query
function burp_escape($s) {
	return db_escape_string($s); 
}

?>